/**********************************************************************
zyGrib: meteorological GRIB file viewer
Copyright (C) 2008 - Jacques Zaninetti - http://www.zygrib.org

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
***********************************************************************/

#include <iostream>
#include <cassert>

#include <QApplication>
#include <QMouseEvent>
#include <QPaintEvent>
#include <QPainter>

#include "Terrain.h"
#include "Orthodromie.h"

//---------------------------------------------------------
// Constructeur
//---------------------------------------------------------
Terrain::Terrain(QWidget *parent, Projection *proj_, GshhsReader *gshhsReader)
    : QWidget(parent)
{
    quality = 0;
    selX0 = selY0 = 0;
    selX1 = selY1 = 0;
    isResizing = true;
    
    timerResize = new QTimer(this);
    assert(timerResize);
    timerResize->setSingleShot(true);
    connect(timerResize, SIGNAL(timeout()), this, SLOT(slotTimerResize()));
    
    timerZoomWheel = new QTimer(this);
    assert(timerZoomWheel);
    timerZoomWheel->setSingleShot(true);
    connect(timerZoomWheel, SIGNAL(timeout()), this, SLOT(slotTimerZoomWheel()));
    deltaZoomWheel = 1.0;
    
	//---------------------------------------------------
	drawer = new MapDrawer(gshhsReader);
	assert(drawer);
	currentMeteoFileType = NOFILE_TYPE;
    
    //----------------------------------------------------------------------------
    showOrthodromie   = Util::getSetting("showOrthodromie", false).toBool();
    duplicateFirstCumulativeRecord = Util::getSetting("duplicateFirstCumulativeRecord", true).toBool();
    interpolateValues = Util::getSetting("interpolateValues", true).toBool();
    windArrowsOnGribGrid = Util::getSetting("windArrowsOnGribGrid", false).toBool();
    
    isEarthMapValid = false;
    mustRedraw = true;

    proj = proj_;
    
    gribPlot = new GribPlot(interpolateValues, windArrowsOnGribGrid);
    assert(gribPlot);
    
    iacPlot = new IacPlot();
    assert(iacPlot);
	
	updateGraphicsParameters();
    setIsobarsStep(drawer->isobarsStep);
    
    isSelectionZoneEnCours = false;
/*    setPalette(QPalette(backgroundColor));
    setAutoFillBackground(true);*/
    
    setMouseTracking(true);
    setFocusPolicy(Qt::StrongFocus);

    
	
	//---------------------------------------------------
	// Define new cross cursor, more visible
	//---------------------------------------------------
/*		* B=1 and M=1 gives black.
		* B=0 and M=1 gives white.
		* B=0 and M=0 gives transparent.
		* B=1 and M=0 gives an XOR'd result.*/
	QBitmap *crossBits = new QBitmap(32,32);
	QBitmap *crossMask = new QBitmap(32,32);
	crossBits->clear();
 	crossMask->clear();
	QPainter pb (crossBits);
	QPainter pm (crossMask);
	QPen pen1(Qt::color1);
	pen1.setWidth(1);
	pb.setPen(pen1);
	pb.drawLine(6,16, 26,16);
	pb.drawLine(16,6, 16,26);
	pm.setPen(pen1);
	 // black
	pm.drawLine(6,16, 26,16);
	pm.drawLine(16,6, 16,26);
	// white
	pm.drawLine(6,15, 26,15);
	pm.drawLine(6,17, 26,17);
	
	pm.drawLine(15,6, 15,26);
	pm.drawLine(17,6, 17,26);
	
	pb.end();
	pm.end();
	myCrossCursor = QCursor(*crossBits, *crossMask);
    
//myCrossCursor=Qt::CrossCursor;
    setCursor(myCrossCursor);
    
}
//-------------------------------------------
void Terrain::updateGraphicsParameters()
{            
    drawer->updateGraphicsParameters();
    int v = 180;
    selectColor     = QColor(v,v,v);
    
	isEarthMapValid = false;
	mustRedraw = true;
    update();
}


//-------------------------------------------------------
// Tracé récursif de la route orthodromique
void Terrain::draw_OrthodromieSegment(QPainter &pnt,
                            double x0,double y0, double x1,double y1,
                            int recurs
                            )
{
    if (recurs > 100)
        return;
    Orthodromie *ortho;
    int i0,j0, i1,j1, im,jm;
    double eps = 0.5;
    if (y0 > 90-eps) y0 = 90-eps;
    if (y0 <-90+eps) y0 =-90+eps;
    if (y1 > 90-eps) y1 = 90-eps;
    if (y1 <-90+eps) y1 =-90+eps;
    
    if (fabs(x0-x1)>180)  // il faut faire le tour du monde par derrière
    {
        if (x0 < x1) {
            draw_OrthodromieSegment(pnt, x1-360,y1, x0,y0, recurs+1);
            draw_OrthodromieSegment(pnt, x0+360,y0, x1,y1, recurs+1);
        }
        else {
            draw_OrthodromieSegment(pnt, x0-360,y0, x1,y1, recurs+1);
            draw_OrthodromieSegment(pnt, x1+360,y1, x0,y0, recurs+1);
        }
    }
    else
    {
        proj->map2screen(x0, y0, &i0, &j0);
        proj->map2screen(x1, y1, &i1, &j1);
        if (abs(i0-i1) > 10)
        {
            double xm, ym;
            
            ortho = new Orthodromie(x0, y0, x1, y1);
            ortho->getMidPoint(&xm, &ym);
            delete ortho;
            ortho = NULL;
            
            xm *= 180.0/M_PI;
            ym *= 180.0/M_PI;
            while (ym > 90)
                ym -= 180;
            while (ym < -90)
                ym += 180;
            proj->map2screen(xm, ym, &im, &jm);
            //printf("%5d: (%5d %5d) (%5d %5d) (%5d %5d)      %f %f   %f %f\n",recurs,i0,j0, im,jm, i1,j1,x0,y0,x1,y1);
            draw_OrthodromieSegment(pnt, x0,y0, xm,ym, recurs+1);
            draw_OrthodromieSegment(pnt, xm,ym, x1,y1, recurs+1);
        }
        else {
                pnt.drawLine(i0,j0, i1,j1);
        }
    }
}

//-------------------------------------------------------
void Terrain::draw_Orthodromie(QPainter &pnt)
{
    draw_OrthodromieSegment(pnt, selX0, selY0, selX1, selY1);
}

//---------------------------------------------------------
void Terrain::indicateWaitingMap()
{
    pleaseWait = true;   // Affiche un message d'attente
    update();
    QCoreApplication::processEvents();
}

//-------------------------------------------------------
void Terrain::setProjection(Projection *proj)
{
//printf("Terrain::setProjection\n");
    indicateWaitingMap();
    this->proj = proj;
    proj->setScreenSize( width(), height());
	
	QList<POI*> lpois = getListPOIs();
	for (int i=0; i<lpois.size(); i++) {
		lpois.at(i)->setProjection(proj);
	}
    isEarthMapValid = false;
	mustRedraw = true;
    update();
}

//=========================================================
void Terrain::setDrawLonLatGrid(bool b) {
    if (drawer->showLonLatGrid != b) {
        drawer->showLonLatGrid = b;
        Util::setSetting("showLonLatGrid", b);
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setDrawRivers(bool b) {
    if (drawer->showRivers != b) {
        drawer->showRivers = b;
        Util::setSetting("showRivers", b);
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::slotTemperatureLabels(bool b) {
    if (drawer->showTemperatureLabels != b) {
        drawer->showTemperatureLabels = b;
        Util::setSetting("showTemperatureLabels", b);
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setDrawCountriesBorders(bool b) {
    if (drawer->showCountriesBorders != b) {
        drawer->showCountriesBorders = b;
        Util::setSetting("showCountriesBorders", b);
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setDrawOrthodromie(bool b) {
    if (showOrthodromie != b) {
        showOrthodromie = b;
        Util::setSetting("showOrthodromie", b);
        mustRedraw = false;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setCountriesNames(bool b) {
    if (drawer->showCountriesNames != b) {
        drawer->showCountriesNames = b;
        Util::setSetting("showCountriesNames", b);
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setCitiesNamesLevel  (int level) {
    if (drawer->showCitiesNamesLevel != level) {
        drawer->showCitiesNamesLevel = level;
        Util::setSetting("showCitiesNamesLevel", level);
        mustRedraw = true;
        update();
    }
}

//-------------------------------------------------------
void Terrain::setMapQuality(int q) {
    indicateWaitingMap();
    if (quality != q) {
        if (drawer->gshhsReader == NULL)
            return;
        quality = q;
        pleaseWait = true;
        //update();
        QCursor oldcursor = cursor();
        setCursor(Qt::WaitCursor);
            drawer->gshhsReader->setUserPreferredQuality(q);
            isEarthMapValid = false;
            update();
        setCursor(oldcursor);
        pleaseWait = false;
    }
}

//-------------------------------------------------------
void Terrain::setColorMapMode(int mode )
{
    if (drawer->colorMapMode != mode)
    {
		Util::setSetting("colorMapMode", mode);
        drawer->colorMapMode = mode;
        mustRedraw = true;
        update();
    }
}

//-------------------------------------------------------
void Terrain::setDuplicateFirstCumulativeRecord (bool b) {
    if (duplicateFirstCumulativeRecord != b) {
        duplicateFirstCumulativeRecord = b;
        Util::setSetting("duplicateFirstCumulativeRecord", b);
	    gribPlot->duplicateFirstCumulativeRecord( b );
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setInterpolateValues (bool b) {
    if (interpolateValues != b) {
        interpolateValues = b;
        Util::setSetting("interpolateValues", b);
	    gribPlot->interpolateValues( b );
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setWindArrowsOnGribGrid (bool b) {
    if (windArrowsOnGribGrid != b) {
        windArrowsOnGribGrid = b;
        Util::setSetting("windArrowsOnGribGrid", b);
	    gribPlot->windArrowsOnGribGrid( b );
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setColorMapSmooth (bool b) {
    if (drawer->colorMapSmooth != b) {
        drawer->colorMapSmooth = b;
        Util::setSetting("colorMapSmooth", b);
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setDrawWindArrows (bool b) {
    if (drawer->showWindArrows != b) {
        drawer->showWindArrows = b;
        Util::setSetting("showWindArrows", b);
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setBarbules (bool b) {
    if (drawer->showBarbules != b) {
        drawer->showBarbules = b;
        Util::setSetting("showBarbules", b);
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setGribGrid (bool b) {
    if (drawer->showGribGrid != b) {
        drawer->showGribGrid = b;
        Util::setSetting("showGribGrid", b);
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setPressureMinMax (bool b) {
    if (drawer->showPressureMinMax != b) {
        drawer->showPressureMinMax = b;
        Util::setSetting("showPressureMinMax", b);
        mustRedraw = true;
        update();
    }
}

//-------------------------------------------------------
void Terrain::setDrawIsobars(bool b) {
    if (drawer->showIsobars != b) {
        drawer->showIsobars = b;
        Util::setSetting("showIsobars", b);
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setIsobarsStep(double step)
{
    if (gribPlot != NULL) {
        gribPlot->setIsobarsStep(step);
        Util::setSetting("isobarsStep", step);
        drawer->isobarsStep = step;
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setDrawIsobarsLabels(bool b) {
    if (drawer->showIsobarsLabels != b) {
        drawer->showIsobarsLabels = b;
        Util::setSetting("showIsobarsLabels", b);
        mustRedraw = true;
        update();
    }
}

//-------------------------------------------------------
void Terrain::setDrawIsotherms0(bool b) {
    if (drawer->showIsotherms0 != b) {
        drawer->showIsotherms0 = b;
        Util::setSetting("showIsotherms0", b);
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setIsotherms0Step(double step)
{
    if (gribPlot != NULL) {
        gribPlot->setIsotherms0Step(step);
        Util::setSetting("isotherms0Step", step);
        drawer->isotherms0Step = step;
        mustRedraw = true;
        update();
    }
}
//-------------------------------------------------------
void Terrain::setDrawIsotherms0Labels(bool b) {
    if (drawer->showIsotherms0Labels != b) {
        drawer->showIsotherms0Labels = b;
        Util::setSetting("showIsotherms0Labels", b);
        mustRedraw = true;
        update();
    }
}

//=================================================================
bool  Terrain::getSelectedRectangle(double *x0, double *y0, double *x1, double *y1)
{
    if (selX0!=selX1 && selY0!=selY1)
    {   // Si nécessaire, réoriente le rectangle sélectionné
        if (selX0 > selX1) {
            *x0 = selX1;
            *x1 = selX0;
        }
        else {
            *x0 = selX0;
            *x1 = selX1;
        }
        if (selY0 > selY1) {
            *y0 = selY0;
            *y1 = selY1;
        }
        else {
            *y0 = selY1;
            *y1 = selY0;
        }
        return true;
    }
    else {
        return false;
    }
}
//---------------------------------------------------------
bool  Terrain::getSelectedLine(double *x0, double *y0, double *x1, double *y1)
{
    *x0 = selX0;
    *x1 = selX1;
    *y0 = selY0;
    *y1 = selY1;
    return true;
}
//---------------------------------------------------------
bool  Terrain::getGribFileRectangle(double *x0, double *y0, double *x1, double *y1)
{
	if (gribPlot) {
		if (gribPlot->isGribReaderOk()) {
			return gribPlot->getGribReader()->getZoneExtension (x0,y0, x1,y1);
		}
	}
    return false;
}

//---------------------------------------------------------
// Grib or Iac files
//---------------------------------------------------------
void Terrain::loadIacFile(QString fileName, bool zoom)
{
	currentMeteoFileType = IAC_TYPE;
    indicateWaitingMap();
    iacPlot->loadIacFile( fileName );
    isSelectionZoneEnCours = false;
    selX0 = selY0 = 0;
    selX1 = selY1 = 0;
    isEarthMapValid = false;
	mustRedraw = true;
    if (zoom) {
        zoomOnIacFile();    // Zoom sur la zone couverte par le fichier GRIB
    }
    update();
}
//---------------------------------------------------------
void Terrain::zoomOnIacFile()
{
//printf("Terrain::zoomOnGribFile\n");
    IacReader * iacReader= iacPlot->getIacReader();
    if (iacReader!=NULL && iacReader->isOk()) {
        double x0,y0, x1,y1, mh, mv;
        if (iacReader->getZoneExtension(&x0,&y0, &x1,&y1))
        {
            mh = fabs(x0-x1)*0.05;
            mv = fabs(y0-y1)*0.05;
            proj->setVisibleArea(x0-mh,y0-mv, x1+mh,y1+mv);
            setProjection(proj);
        }
    }
}

//---------------------------------------------------------
void Terrain::loadGribFile(QString fileName, bool zoom)
{
	currentMeteoFileType = GRIB1_TYPE;
    indicateWaitingMap();
    
    gribPlot->loadGribFile( fileName );
    gribPlot->duplicateFirstCumulativeRecord( duplicateFirstCumulativeRecord );
    
    isSelectionZoneEnCours = false;
    selX0 = selY0 = 0;
    selX1 = selY1 = 0;
    isEarthMapValid = false;
	mustRedraw = true;
    if (zoom) {
        zoomOnGribFile();    // Zoom sur la zone couverte par le fichier GRIB
    }
    update();
}
//---------------------------------------------------------
void Terrain::zoomOnGribFile()
{
	double x0,y0, x1,y1, mh, mv;
    if (currentMeteoFileType == GRIB1_TYPE)
    {
		GribReader * gribReader= gribPlot->getGribReader();
		if (gribReader != NULL) {
			if (gribReader->getZoneExtension(&x0,&y0, &x1,&y1))
			{
				mh = fabs(x0-x1)*0.05;
				mv = fabs(y0-y1)*0.05;
				proj->setVisibleArea(x0-mh,y0-mv, x1+mh,y1+mv);
				setProjection(proj);
			}
		}
    }
    else if (currentMeteoFileType == IAC_TYPE) {
		IacReader * iacReader= iacPlot->getIacReader();
		if (iacReader!=NULL && iacReader->isOk()) {
			if (iacReader->getZoneExtension(&x0,&y0, &x1,&y1))
			{
				mh = fabs(x0-x1)*0.05;
				mv = fabs(y0-y1)*0.05;
				proj->setVisibleArea(x0-mh,y0-mv, x1+mh,y1+mv);
				setProjection(proj);
			}
		}
    }
}
//---------------------------------------------------------
void Terrain::slotMustRedraw()
{
    indicateWaitingMap();
    isEarthMapValid = false;
	mustRedraw = true;
    update();
}
//---------------------------------------------------------
void Terrain::setCurrentDate(time_t t)
{
    if (gribPlot->getCurrentDate() != t)
    {
        indicateWaitingMap();
        gribPlot->setCurrentDate(t);
		mustRedraw = true;
        update();
    }
}


//---------------------------------------------------------
// Events
//---------------------------------------------------------
//---------------------------------------------------------
void Terrain::enterEvent (QEvent * /*e*/) {
//printf("enter\n");
    enterCursor = cursor();
    setCursor(myCrossCursor);
}
//---------------------------------------------------------
void Terrain::leaveEvent (QEvent * /*e*/) {
//printf("leave\n");
    setCursor(enterCursor);
}

//---------------------------------------------------------
void  Terrain::keyPressEvent (QKeyEvent *e)
{
//printf("Terrain::keyPressEvent\n");
	keyModifiers = e->modifiers();
    if (keyModifiers == Qt::ControlModifier) {
        setCursor(Qt::SizeAllCursor);
    }
    else if (keyModifiers == Qt::ShiftModifier) {
        setCursor(Qt::UpArrowCursor);
    }
    else {
        setCursor(myCrossCursor);
    }
}
//---------------------------------------------------------
void  Terrain::keyReleaseEvent (QKeyEvent *e)
{
//printf("keyReleaseEvent\n");
	keyModifiers = e->modifiers();
    if (keyModifiers == Qt::ControlModifier) {
        setCursor(Qt::SizeAllCursor);
    }
    else if (keyModifiers == Qt::ShiftModifier) {
        setCursor(Qt::UpArrowCursor);
    }
    else {
        setCursor(myCrossCursor);
    }
}
//---------------------------------------------------------
void  Terrain::wheelEvent(QWheelEvent * e)
{
//printf("wheelEvent\n");
	double k = 1.3;
	
	if (e->delta() > 0)
		deltaZoomWheel /= k;
	else
		deltaZoomWheel *= k;
	
	// Le timer évite les multiples update() pendant les changements de taille
    timerZoomWheel->stop();	 // pas d'update() tout de suite
    timerZoomWheel->start(150); // update() seulement après une petite inactivité
}
//---------------------------------------------------------
void Terrain::slotTimerZoomWheel () {
    if (deltaZoomWheel != 1) {
		//printf("slotTimerZoomWheel\n");
		proj->zoom(deltaZoomWheel);
		deltaZoomWheel = 1;
		setProjection(proj);
    }
}


//---------------------------------------------------------
void Terrain::mousePressEvent (QMouseEvent * e) {
//printf("press\n");
    if (e->button() == Qt::LeftButton)
    {
        // Début de sélection de zone rectangulaire
        isSelectionZoneEnCours = true;
        proj->screen2map(e->x(),e->y(), &selX1, &selY1);
/*        selX1 = floor(selX1*12)/12;
        selY1 = ceil (selY1*12)/12;*/
        selX0 = selX1;
        selY0 = selY1;
        update();
    }
}
//---------------------------------------------------------
void Terrain::mouseReleaseEvent (QMouseEvent * e) {
//printf("release\n");
    double x0, y0, x1, y1;
/*proj->screen2map(e->x(), e->y(), &x0,&y0);
printf("clic: i=%d j=%d   x=%f y=%f\n", e->x(), e->y(), x0, y0);*/
    
    if (isSelectionZoneEnCours)
    {
        isSelectionZoneEnCours = false;
        proj->screen2map(e->x(),e->y(), &selX1, &selY1);
/*        selX1 = ceil (selX1*12)/12;
        selY1 = floor(selY1*12)/12;*/
        if (getSelectedRectangle(&x0,&y0, &x1,&y1))
        {
            emit selectionOK(x0, y0, x1, y1);
            // Zoom sur la sélection si Control appuyé
            if (e->modifiers() == Qt::ControlModifier) {
                this->slot_Zoom_Sel();
            }
        }
        else {
            emit mouseClicked(e);
        }
    }
    else {
        emit mouseClicked(e);
    }
}

//---------------------------------------------------------
void Terrain::mouseMoveEvent (QMouseEvent * e) {
//printf("move %d %d\n",e->x(),e->y());
    if (isSelectionZoneEnCours)
    {
        proj->screen2map(e->x(),e->y(), &selX1, &selY1);
/*        selX1 = ceil (selX1*12)/12;
        selY1 = floor(selY1*12)/12;*/
        update();
    }
    emit mouseMoved(e);
}

//---------------------------------------------------------
void Terrain::resizeEvent (QResizeEvent * /*e*/)
{
	static int cpt = 0;
	cpt ++;

//printf("resize %03d : %d %d\n", cpt, width(), height());

    indicateWaitingMap();

	int W = width();
	int H = height();
	double x, y;
	proj->screen2map(0, 0, &x, &y);	// position du coin haut-gauche
    
    proj->setScreenSize(W, H);
    
    if (cpt > 1) {   // pas de repositionnement lors de l'ouverture (1er resize)
    	proj->setMapPointInScreen(x,y, 0,0); // coin haut-gauche fixe
    }
    
    isEarthMapValid = false;
	mustRedraw = true;
    isResizing = true;
	
	// Le timer évite les multiples update() pendant les changements de taille
    timerResize->stop();	 // pas d'update() tout de suite
    timerResize->start(100); // update() seulement après une petite inactivité
}
//---------------------------------------------------------
void Terrain::slotTimerResize () {
    if (isResizing) {
//printf("timer update\n");
        isResizing = false;
        update();
    }
}
//---------------------------------------------------------
void Terrain::slot_Zoom_In()
{
    proj->zoom(1.3);
    setProjection(proj);
}
void Terrain::slot_Zoom_Out()
{
    proj->zoom(0.7);
    setProjection(proj);
}
//---------------------------------------------------------
void Terrain::slot_Zoom_Sel()
{
    double x0, y0, x1, y1;
    if (getSelectedRectangle(&x0,&y0, &x1,&y1))
    {
        // zoom sur la zone sélectionnée
        proj->setVisibleArea(x0,y0, x1,y1);
        isSelectionZoneEnCours = false;
        selX0 = selY0 = 0;
        selX1 = selY1 = 0;
    }
    else {
        indicateWaitingMap();
        selX0 = selY0 = 0;
        selX1 = selY1 = 0;
        isSelectionZoneEnCours = false;
        isEarthMapValid = false;
		mustRedraw = true;
        zoomOnGribFile();
        update();
    }
	setProjection(proj);
}
//---------------------------------------------------------
void Terrain::slot_Zoom_All()
{
	proj->setVisibleAreaEarth();
    setProjection(proj);
}
//------------------------------------------------
void Terrain::slot_Go_Left()
{
    proj->move( 0.25, 0);
    setProjection(proj);
}
void Terrain::slot_Go_Right()
{
    proj->move(-0.25, 0);
    setProjection(proj);
}
void Terrain::slot_Go_Up()
{
    proj->move(0,  -0.25);
    setProjection(proj);
}
void Terrain::slot_Go_Down()
{
    proj->move(0,  0.25);
    setProjection(proj);
}

//---------------------------------------------------------
void Terrain::setShowPOIs(bool show)
{
	Util::setSetting("showPOIs", show);
	// list of all the POI's
	QList<POI*> lpois = getListPOIs();
	for (int i=0; i<lpois.size(); i++)
	{
		if (show)
			lpois.at(i)->setVisible(true);
		else
			lpois.at(i)->setVisible(false);
	}
}

//---------------------------------------------------------
// paintEvent
//---------------------------------------------------------
void Terrain::paintEvent(QPaintEvent * /*event*/)
{
    QPainter pnt(this);
    QColor transp;
    int r = 100;
    if (!isResizing)
    {
        // Draw the map and the GRIB data
        QCursor oldcursor = cursor();
        setCursor(Qt::WaitCursor);
        
        if (currentMeteoFileType == GRIB1_TYPE) {
        	drawer->draw_GSHHSandGRIB1 (pnt, mustRedraw, isEarthMapValid, proj, gribPlot);
        }
        else if (currentMeteoFileType == IAC_TYPE) {
        	drawer->draw_GSHHSandIAC (pnt, mustRedraw, isEarthMapValid, proj, iacPlot);
        }
		else {
        	drawer->draw_GSHHS (pnt, mustRedraw, isEarthMapValid, proj);
		}
		
        setCursor(oldcursor);
		isEarthMapValid = true;
		mustRedraw = false;
		pleaseWait = false;
        
        if (selX0!=selX1 && selY0!=selY1) {
            // Draw the rectangle of the selected zone
            pnt.setPen(selectColor);
            transp = QColor(r,r,r, 80);
            pnt.setBrush(transp);
            int x0,y0, x1,y1;
            proj->map2screen(selX0, selY0, &x0, &y0);
            proj->map2screen(selX1, selY1, &x1, &y1);
            pnt.drawRect(x0, y0, x1-x0, y1-y0);
            
            if (showOrthodromie)
            {
                QPen penLine(QColor(Qt::white));
                penLine.setWidthF(1.6);
                pnt.setPen(penLine);
                draw_Orthodromie(pnt);
            }
        }

    }
    
    if (pleaseWait) {
        // Write the message "please wait..." on the map
        QFont fontWait("Helvetica", 12, QFont::Bold, true);
        QFontMetrics fmet(fontWait);
        pnt.setPen(QColor(Qt::white));
        r = 80;
        transp = QColor(r,r,r, 80);
        pnt.setFont(fontWait);
        pnt.setBrush(transp);
        QString txt = tr("  Calculs en cours...  ");
        QRect rect = fmet.boundingRect(txt);

        rect.moveTo(20,20);
        pnt.drawRect(rect);
        pnt.drawText(rect, Qt::AlignHCenter|Qt::AlignVCenter , txt);
    }

// 	bool showCenter = true;
//     if (showCenter) {
//     	int s = 20;
//         pnt.setPen(QColor(Qt::red));
// 		pnt.drawLine(width()/2-s,height()/2, width()/2+s,height()/2);
// 		pnt.drawLine(width()/2,height()/2-s, width()/2,height()/2+s);
//     }

}



